/**
 * Module dependencies
 */

var http = require('http')
  , fs = require('fs')
  , path = require('path');

var config = require(path.join(process.env.PWD, 'config', 'application'))
  , controllers = collectControllers()
  , routes = require(path.join(process.env.PWD, 'config', 'routes'))
  , _index = 0
  , _req
  , _res
  , _out;

function collectControllers() {
  var cons = fs.readdirSync(path.join(process.env.PWD, config.paths.controllers))
    , obj = {}
    , len = cons.length
    , i = 0;
  for (i; i < len; i += 1) {
    var name = path.basename(cons[i], '.js');
    obj[name] = require(path.join(process.env.PWD, config.paths.controllers, cons[i]));
  }
  return obj;
}

function collectMiddleware() {
  var layers = [];
  config.middleware.forEach(function (layer) {
    var name = Object.keys(layer)[0]
      , mw = require('./middleware/' + name);
    layers.push({ 
      handle: mw(layer[name].options) 
    });
  });
  return layers;
}

function next(err) {
  var layer = application.middleware[_index]
    , arity;

  _index += 1;

  // completed middleware stack
  if (!layer) {
    if (_out) _out(err);
    if (err) {
      _res.end();
    } else if (_res.headersSent) {
      _res.end();
    } else {
      _res.statusCode = 404;
      _res.setHeader('Content-Type', 'text/html');
      _res.end('404 not found');
    }
    return;
  }

  arity = layer.handle.length;

  if (err && arity === 4) {
    layer.handle(err, _req, _res, next);
  } else if (err && arity < 4) {
    next(err);
  } else if (!err && arity < 4) {
    layer.handle(_req, _res, next);
  } else {
    next();
  }
}

var application = function application() {
  this.config = config;
  this.controllers = controllers;
  this.middleware = application.middleware = collectMiddleware();
  this.routes = routes;

  if (application.caller !== application.getInstance) {
    throw new Error("This object cannot be instantiated.");
  }
};

application.instance = null;

application.getInstance = function () {
  if (this.instance === null) {
    this.instance = new application();
  }
  return this.instance;
};

application.prototype.handle = function (req, res, out) {
  _index = 0;
  _req = req;
  _res = res;
  _out = out;
  next();
};

application.prototype.listen = function () {
  var server = http.createServer(this.handle);
  return server.listen.apply(server, arguments);
};

module.exports = application.getInstance();
